@@ -24,6 +24,8 @@
 
 #include <gtk/gtk.h>
 #include "gdk-pixbuf.h"
+#include "gdk-pixbuf-io.h"
+#include "gdk-pixbuf-loader.h"
 
 #define DEFAULT_WIDTH  24
 #define DEFAULT_HEIGHT 24
@@ -175,6 +177,126 @@ static const unsigned char default_image[] = {
 	0xff, 0x00, 0xff, 0xff, 0x00, 0xff, 0xff, 0x00, 0xff, 0xff, 0x00, 0xff
 };
 
+
+static const char * book_open_xpm[] = {
+"16 16 4 1",
+"       c None s None",
+".      c black",
+"X      c #808080",
+"o      c white",
+"                ",
+"  ..            ",
+" .Xo.    ...    ",
+" .Xoo. ..oo.    ",
+" .Xooo.Xooo...  ",
+" .Xooo.oooo.X.  ",
+" .Xooo.Xooo.X.  ",
+" .Xooo.oooo.X.  ",
+" .Xooo.Xooo.X.  ",
+" .Xooo.oooo.X.  ",
+"  .Xoo.Xoo..X.  ",
+"   .Xo.o..ooX.  ",
+"    .X..XXXXX.  ",
+"    ..X.......  ",
+"     ..         ",
+"                "};
+
+static const char * book_closed_xpm[] = {
+"16 16 6 1",
+"       c None s None",
+".      c black",
+"X      c red",
+"o      c yellow",
+"O      c #808080",
+"#      c white",
+"                ",
+"       ..       ",
+"     ..XX.      ",
+"   ..XXXXX.     ",
+" ..XXXXXXXX.    ",
+".ooXXXXXXXXX.   ",
+"..ooXXXXXXXXX.  ",
+".X.ooXXXXXXXXX. ",
+".XX.ooXXXXXX..  ",
+" .XX.ooXXX..#O  ",
+"  .XX.oo..##OO. ",
+"   .XX..##OO..  ",
+"    .X.#OO..    ",
+"     ..O..      ",
+"      ..        ",
+"                "};
+
+static const char * mini_page_xpm[] = {
+"16 16 4 1",
+"       c None s None",
+".      c black",
+"X      c white",
+"o      c #808080",
+"                ",
+"   .......      ",
+"   .XXXXX..     ",
+"   .XoooX.X.    ",
+"   .XXXXX....   ",
+"   .XooooXoo.o  ",
+"   .XXXXXXXX.o  ",
+"   .XooooooX.o  ",
+"   .XXXXXXXX.o  ",
+"   .XooooooX.o  ",
+"   .XXXXXXXX.o  ",
+"   .XooooooX.o  ",
+"   .XXXXXXXX.o  ",
+"   ..........o  ",
+"    oooooooooo  ",
+"                "};
+
+static const char * gtk_mini_xpm[] = {
+"15 20 17 1",
+"       c None",
+".      c #14121F",
+"+      c #278828",
+"@      c #9B3334",
+"#      c #284C72",
+"$      c #24692A",
+"%      c #69282E",
+"&      c #37C539",
+"*      c #1D2F4D",
+"=      c #6D7076",
+"-      c #7D8482",
+";      c #E24A49",
+">      c #515357",
+",      c #9B9C9B",
+"'      c #2FA232",
+")      c #3CE23D",
+"!      c #3B6CCB",
+"               ",
+"      ***>     ",
+"    >.*!!!*    ",
+"   ***....#*=  ",
+"  *!*.!!!**!!# ",
+" .!!#*!#*!!!!# ",
+" @%#!.##.*!!$& ",
+" @;%*!*.#!#')) ",
+" @;;@%!!*$&)'' ",
+" @%.%@%$'&)$+' ",
+" @;...@$'*'*)+ ",
+" @;%..@$+*.')$ ",
+" @;%%;;$+..$)# ",
+" @;%%;@$$$'.$# ",
+" %;@@;;$$+))&* ",
+"  %;;;@+$&)&*  ",
+"   %;;@'))+>   ",
+"    %;@'&#     ",
+"     >%$$      ",
+"      >=       "};
+
+const gchar ** xpms[] = { 
+  book_open_xpm,
+  book_closed_xpm,
+  mini_page_xpm,
+  gtk_mini_xpm,
+  NULL
+};
+
 static void
 quit_func (GtkWidget *widget, gpointer dummy)
 {
@@ -188,6 +310,11 @@ expose_func (GtkWidget *drawing_area, GdkEventExpose *event, gpointer data)
 
 	pixbuf = (GdkPixbuf *)gtk_object_get_data(GTK_OBJECT(drawing_area), "pixbuf");
 
+	if (!pixbuf->art_pixbuf) {
+		g_warning ("art_pixbuf is NULL in expose_func!!\n");
+		return;
+	}
+
 	if (pixbuf->art_pixbuf->has_alpha) {
 		gdk_draw_rgb_32_image (drawing_area->window,
 				       drawing_area->style->black_gc,
@@ -211,7 +338,7 @@ expose_func (GtkWidget *drawing_area, GdkEventExpose *event, gpointer data)
 				    + (event->area.x * pixbuf->art_pixbuf->n_channels),
 				    pixbuf->art_pixbuf->rowstride);
 	}
-}  
+}
 
 static void
 config_func (GtkWidget *drawing_area, GdkEventConfigure *event, gpointer data)
@@ -229,11 +356,12 @@ config_func (GtkWidget *drawing_area, GdkEventConfigure *event, gpointer data)
 #endif
 }
 
-static void
-new_testrgb_window (GdkPixbuf *pixbuf)
+static GtkWidget*
+new_testrgb_window (GdkPixbuf *pixbuf, gchar *title)
 {
 	GtkWidget *window;
 	GtkWidget *vbox;
+	GtkWidget *temp_box;
 	GtkWidget *button;
 	GtkWidget *drawing_area;
 	gint w, h;
@@ -252,10 +380,17 @@ new_testrgb_window (GdkPixbuf *pixbuf)
 
 	vbox = gtk_vbox_new (FALSE, 0);
 
+	if (title)
+		gtk_box_pack_start (GTK_BOX (vbox), gtk_label_new (title),
+				    TRUE, TRUE, 0);
+
 	drawing_area = gtk_drawing_area_new ();
 
+	temp_box = gtk_hbox_new (FALSE, 0);
 	gtk_drawing_area_size (GTK_DRAWING_AREA(drawing_area), w, h);
-	gtk_box_pack_start (GTK_BOX (vbox), drawing_area, TRUE, TRUE, 0);
+	gtk_box_pack_start (GTK_BOX (temp_box), drawing_area, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (vbox), temp_box, FALSE, FALSE, 0);
+	
 
 	gtk_signal_connect (GTK_OBJECT(drawing_area), "expose_event",
 			    GTK_SIGNAL_FUNC(expose_func), NULL);
@@ -275,18 +410,60 @@ new_testrgb_window (GdkPixbuf *pixbuf)
 	gtk_widget_show (button);
 
 	gtk_container_add (GTK_CONTAINER (window), vbox);
-	gtk_widget_show (vbox);
+	gtk_widget_show_all (vbox);
 
 	gtk_widget_show (window);
+
+        return window;
 }
 
+static gint
+update_timeout(gpointer data)
+{
+        GtkWidget** window_loc = data;
+
+        if (*window_loc != NULL) {
+                gtk_widget_queue_draw(*window_loc);
+        }
+
+	return TRUE;
+}
+
+static void
+progressive_prepared_callback(GdkPixbufLoader* loader, gpointer data)
+{
+        GtkWidget** retloc = data;
+        GdkPixbuf* pixbuf;
+
+        pixbuf = gdk_pixbuf_loader_get_pixbuf(loader);
+        g_assert(pixbuf != NULL);
+
+        gdk_pixbuf_ref(pixbuf); /* for the RGB window */
+
+        *retloc = new_testrgb_window(pixbuf, "Progressive");
+
+        return;
+}
+
+
+static void
+progressive_updated_callback(GdkPixbufLoader* loader, guint x, guint y, guint width, guint height, gpointer data)
+{
+	g_print ("progressive_updated_callback:\n\t%d\t%d\t%d\t%d\n", x, y, width, height);
+        return;
+}
+
+static int readlen = 1;
+
 int
 main (int argc, char **argv)
 {
 	int i;
-	int found_valid = FALSE; 
+	int found_valid = FALSE;
 
 	GdkPixbuf *pixbuf;
+	GdkPixbufLoader *pixbuf_loader;
+	FILE *file;
 
 	gtk_init (&argc, &argv);
 
@@ -297,25 +474,88 @@ main (int argc, char **argv)
 	gtk_widget_set_default_colormap (gdk_rgb_get_cmap ());
 	gtk_widget_set_default_visual (gdk_rgb_get_visual ());
 
+	{
+		char *tbf_readlen = getenv("TBF_READLEN");
+		if(tbf_readlen) readlen = atoi(tbf_readlen);
+	}
+
 	i = 1;
 	if (argc == 1) {
+                const gchar*** xpmp;
+                
 		pixbuf = gdk_pixbuf_new_from_data ((guchar *) default_image, ART_PIX_RGB, FALSE,
 						   DEFAULT_WIDTH, DEFAULT_HEIGHT, DEFAULT_WIDTH * 3,
 						   NULL, NULL);
-		new_testrgb_window (pixbuf);
+		new_testrgb_window (pixbuf, NULL);
+
+                xpmp = xpms;
+                while (*xpmp) {
+                        pixbuf = gdk_pixbuf_new_from_xpm_data (*xpmp);
+                        new_testrgb_window (pixbuf, NULL);
+                        ++xpmp;
+                }
+                
 		found_valid = TRUE;
 	} else {
 		for (i = 1; i < argc; i++) {
+
 			pixbuf = gdk_pixbuf_new_from_file (argv[i]);
 #if 0
 			pixbuf = gdk_pixbuf_rotate(pixbuf, 10.0);
 #endif
 
 			if (pixbuf) {
-				new_testrgb_window (pixbuf);
+				new_testrgb_window (pixbuf, "File");
 				found_valid = TRUE;
 			}
 		}
+
+                {
+                        GtkWidget* rgb_window = NULL;
+                        guint timeout;
+			char *buf;
+
+                        pixbuf_loader = gdk_pixbuf_loader_new ();
+
+                        gtk_signal_connect(GTK_OBJECT(pixbuf_loader),
+                                           "area_prepared",
+                                           GTK_SIGNAL_FUNC(progressive_prepared_callback),
+                                           &rgb_window);
+
+                        gtk_signal_connect(GTK_OBJECT(pixbuf_loader),
+                                           "area_updated",
+                                           GTK_SIGNAL_FUNC(progressive_updated_callback),
+                                           &rgb_window);
+
+                        timeout = gtk_timeout_add(1000, update_timeout, &rgb_window);
+                        
+                        file = fopen (argv[1], "r");
+                        g_assert (file != NULL);
+			buf = g_malloc(readlen);
+
+                        while (!feof(file)) {
+				int nbytes;
+				nbytes = fread(buf, 1, readlen, file);
+
+                                //printf(".");
+                                fflush(stdout);
+                                
+                                if (gdk_pixbuf_loader_write (GDK_PIXBUF_LOADER (pixbuf_loader), buf, nbytes) == FALSE)
+                                        break;
+
+	                        while (gtk_events_pending())
+	                        	gtk_main_iteration();
+                        }
+			printf("\n");
+			gtk_timeout_remove (timeout);
+                        gdk_pixbuf_loader_close (GDK_PIXBUF_LOADER (pixbuf_loader));
+			gtk_object_destroy (GTK_OBJECT(pixbuf_loader));
+                        fclose (file);
+
+                        if (rgb_window != NULL)
+                                gtk_widget_queue_draw(rgb_window);
+                }
+		
 	}
 
 	if (found_valid)
